package tp4509.picross;

import java.security.InvalidParameterException;

/**
 * Gestion des couleurs pour un Picross bi-color (noir, blanc).
 * @author Eric Lallet.
 *
 */
public enum PicrossColor {
	/**
	 * Case sans couleur (blanche).
	 */
	NONE('-'),
	/**
	 * Case avec couleur (noire).
	 */
	BLACK('#');
	
	/**
	 * caractère associée à la couleur dans le fichier de solution.
	 */
	private char car;

	/**
	 * constructeur qui initialise car.
	 * @param car
	 *        le caractère associé à la couleur.
	 */
	PicrossColor(final char car) {
		this.car = car;
	}

	/**
	 * Retrouve la couleur par le caractère associé.
	 * @param car
	 *        le caractère associé.
	 * @return
	 *        la couleur pour ce caractère.
	 */
	static PicrossColor colorByChar(final char car) {
		for (PicrossColor col: PicrossColor.values()) {
			if (col.car == car) {
				return col;
			}
		}
		throw new InvalidParameterException("No color is defined for [" + car + "].");
	}
	
	/**
	 * getter sur le caractère associé.
	 * @return
	 *        le caractère associé de la couleur.
	 */
	public char getCar() {
		return car;
	}
}
