//CHECKSTYLE:OFF
package tp4509.tcp;

import static dm4509.common.Log.LOGGER_NAME_TEST;
import static dm4509.common.Log.LOGGER_NAME_COMM;
import static dm4509.common.Log.LOG_ON;
import static dm4509.common.Log.COMM;
import static dm4509.common.Log.TEST;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.FileChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import org.apache.log4j.Level;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;


import dm4509.common.Log;
import dm4509.tcpservices.TcpTestTools;
import tp4509.picross.FileFormatException;
import tp4509.picross.PicrossApi;

public class TestThreadClient {
	static private TcpTestTools tcpTestTools;
	static private PicrossApi picrossApi;

	@BeforeClass
	static public void lanceTestTools() throws InterruptedException {
		tcpTestTools = new TcpTestTools();
		Log.configureALogger(LOGGER_NAME_TEST, Level.TRACE);
		Log.configureALogger(LOGGER_NAME_COMM, Level.INFO);
		try {
			picrossApi = new PicrossApi("./data/poisson.txt");
		} catch (IOException | FileFormatException e) {
			e.printStackTrace();
		}
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient] ----------- Début des tests unitaires pour la classe ThreadClient -----------");
		}
	}

	
	@AfterClass
	static public void finTest() throws InterruptedException {
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient] ----------- Fin des tests unitaires pour la classe ThreadClient -----------\n");
		}
	}
	//@Ignore
	@Test
	public void testRecevoirMessage() throws Exception {

		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient::testRecevoirMessage] ->  test de la réception des messages");
		}

		// emission du message par un serveur.
		MessageProtocole message = new MessageProtocole(Protocole.INDICE_LIGNE, 3, 2, 1);
		FileOutputStream fout = new FileOutputStream("./tmp/message.data");
		FileChannel fcout = fout.getChannel();
		fcout.write(message.getBuffer());
		fcout.close();
		fout.close();


		// lancement d'un serveur de test TCP
		tcpTestTools.serverSendFileStart(40001, "./tmp/message.data");

		// création d'un client
		SocketChannel rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName("localhost"),  40001));
		ThreadClient client = new ThreadClient(rwChan, picrossApi);
		MessageProtocole message2 = client.recevoirMessage();

		rwChan.close();


		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRecevoirMessage] -> message reçu = message émis?");
		}
		assertTrue("Message reçu identique à message envoyé", message.equals(message2));
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient::testRecevoirMessage] -> Fin du test");
		}

	}

	//@Ignore
	@Test
	public void testRun() throws Exception {
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient::testRun] ->  tests du protocole du client");
		}


		// création d'un serveur
		ServerSocketChannel listenChannel;
		listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(40001));

		// création d'un client
		SocketChannel rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName("localhost"),  40001));
		PicrossApi picrossApi = new PicrossApi(rwChan); 
		picrossApi.indiquerInitialisationFinie();

		// lancement du Thread client
		Thread threadClient = new Thread(new ThreadClient(rwChan, picrossApi)); 
		threadClient.start();
		SocketChannel client = listenChannel.accept();

		// Test TAILLE_GRILLE
		// largeur = 10 , ey hauteur = 5
		MessageProtocole message = new MessageProtocole(Protocole.TAILLE_GRILLE, 10, 5, 0); 
		client.write(message.getBuffer());
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.
		int largeur = picrossApi.getLargeur();
		int hauteur = picrossApi.getHauteur();
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> message TAILLE_GRILLE reçu et traité ? " + largeur + " x " + hauteur );
		}
		assertTrue("La largeur et la hauteur de la grille correctement initialisées.", largeur == 10 && hauteur == 5);


		// Test TAILLE_INDICES
		// largeurIndices = 3 , et hauteurIndices = 2
		message = new MessageProtocole(Protocole.TAILLE_INDICES, 3, 2, 0); 
		client.write(message.getBuffer());
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.
		int largeurIndices = picrossApi.getLargeurIndices();
		int hauteurIndices = picrossApi.getHauteurIndices();
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> message TAILLE_INDICE reçu et traité ? " + largeurIndices + " x " + hauteurIndices );
		}
		assertTrue("La largeur et la hauteur des indices correctement initialisées.", largeurIndices == 3 && hauteurIndices == 2);

		// Test INDICE_COLONNE
		// case 5 x 1 = 3
		message = new MessageProtocole(Protocole.INDICE_COLONNE, 5, 1, 3); 
		client.write(message.getBuffer());
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.
		int valeur = picrossApi.getIndicesColonne(5, 1);
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> message INDICE_COLONNE reçu et traité ? " + valeur );

		}
		assertTrue("L'indice colonne  correctement initialisé.", valeur == 3);

		// Test INDICE_LIGNE
		// case 1 x 3 = 4
		message = new MessageProtocole(Protocole.INDICE_LIGNE, 1, 3, 4); 
		client.write(message.getBuffer());
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.
		valeur = picrossApi.getIndicesLigne(1, 3);
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> message INDICE_LIGNE reçu et traité ? " + valeur );

		}
		assertTrue("L'indice ligne  correctement initialisé.", valeur == 4);


		// Test VALEUR_GRILLE_SERVEUR
		// case 6 x 4 = 1
		message = new MessageProtocole(Protocole.VALEUR_GRILLE_SERVEUR, 6, 4, 1); 
		client.write(message.getBuffer());
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.
		valeur = picrossApi.getValeur(6, 4);
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> message VALEUR_GRILLE_SERVEUR reçu et traité ? " + valeur );

		}
		assertTrue("Valeur de grille correctement initialisée.", valeur == 1);

		// Test GAGNE
		message = new MessageProtocole(Protocole.GAGNE, 0, 0, 0); 
		client.write(message.getBuffer());
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.
		boolean gagne = picrossApi.estGagne();
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> message GAGNE reçu et traité ? " + gagne );

		}
		assertTrue("Status gagné passé à vrai.", gagne);


		// Le Thread doit être terminé
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> thread terminé ? " + !threadClient.isAlive() );

		}
		assertFalse("Le thread doit être terminé" , threadClient.isAlive());

		client.close();
		listenChannel.close();
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient::testRun] -> Fin du test");
		}
	}


	//@Ignore
	@Test
	public void testRun2() throws Exception {
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient::testRun2] ->  tests de robusteste");
		}


		// création d'un serveur
		ServerSocketChannel listenChannel;
		listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(40001));

		// création d'un client
		SocketChannel rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName("localhost"),  40001));
		PicrossApi picrossApi = new PicrossApi(rwChan); 
		picrossApi.indiquerInitialisationFinie();

		// lancement du Thread client
		Thread threadClient = new Thread(new ThreadClient(rwChan, picrossApi)); 
		threadClient.start();
		SocketChannel client = listenChannel.accept();

		// Test TAILLE_INDICE en premier message
		MessageProtocole message = new MessageProtocole(Protocole.TAILLE_INDICES, 3, 4, 0); 
		client.write(message.getBuffer());
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.


		// Le Thread doit être terminé
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> thread terminé après erreur fatale de protocole? " + !threadClient.isAlive() );

		}
		assertFalse("Le thread doit être terminé" , threadClient.isAlive());
		client.close();
		
		
		

		// création d'un nouveau client
		rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName("localhost"),  40001));


		// lancement du Thread client
		threadClient = new Thread(new ThreadClient(rwChan, picrossApi)); 
		threadClient.start();
		client = listenChannel.accept();

		// Test d'une deconnexion intempestive 
		client.close();
		Thread.sleep(1000); // on laisse le temps au client de recevoir et gérer le message.

	
		// Le Thread doit être terminé
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("[TestThreadClient::testRun] -> thread terminé après deconnexion serveur? " + !threadClient.isAlive() );

		}
		assertFalse("Le thread doit être terminé" , threadClient.isAlive());
		
		
		
		listenChannel.close();
		
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("[TestThreadClient::testRun2] ->  Fin du test");
		}
	}





}
